/***************************************************************************
 *   Copyright (C) 2005 by Thorsten Schnebeck   *
 *   thorsten.schnebeck@gmx.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <config.h>
#include <kgenericfactory.h>
#include <ktempfile.h>
#include <qfile.h>
#include "kfile_rawimage.h"

extern "C" {
  #include <INTERN.h>
  #include <EXTERN.h>
  #include <perl.h>
  static PerlInterpreter *my_perl;
  void xs_init (pTHX);
}

typedef KGenericFactory<rawimagePlugin> rawimageFactory;

K_EXPORT_COMPONENT_FACTORY(kfile_rawimage, rawimageFactory( "kfile_rawimage" ))

rawimagePlugin::rawimagePlugin(QObject *parent, const char *name, const QStringList &args)
  : KFilePlugin(parent, name, args) {

  kdDebug(7034) << "rawimage plugin\n";
  QPtrList<KFileMimeTypeInfo> info; 
  info.append(addMimeTypeInfo("image/x-raw"));
  info.append(addMimeTypeInfo("image/jpeg"));
  info.append(addMimeTypeInfo("image/png"));

  KFileMimeTypeInfo::GroupInfo *group;
  KFileMimeTypeInfo::GroupInfo *iptc;
  KFileMimeTypeInfo::ItemInfo *item;

  for ( KFileMimeTypeInfo *i = info.first(); i; i = info.next()) {
    // our new group
    group = addGroupInfo(i, "INFO"   , i18n("Image Information"));
    iptc  = addGroupInfo(i, "IPTCHeader", i18n("IPTC Header"));
    // our new items in the group
    addItemInfo( group, "01CreationDate", i18n("Creation Date"),
                 QVariant::Date );

    addItemInfo( group, "02FocalLength", i18n("Focal Length"),
                 QVariant::String );

    addItemInfo( group, "03Aperture", i18n("Aperture"),
                 QVariant::String );

    addItemInfo( group, "04ExposureTime", i18n("Exposure Time"),
                 QVariant::String );
  
    addItemInfo( group, "05LensType", i18n("Lens"),
                 QVariant::String );
  
    addItemInfo( group, "06Iso", i18n("ISO"),
                 QVariant::String );
  
    addItemInfo( group, "07FlashUsed", i18n("Flash"),
                 QVariant::String );
  
    item = addItemInfo( group, "08Dimensions", i18n("Dimensions"),
                        QVariant::Size );
    setHint( item, KFileMimeTypeInfo::Size );
    setUnit( item, KFileMimeTypeInfo::Pixels );
  
    addItemInfo( group, "09Manufacturer", i18n("Camera Manufacturer"),
                 QVariant::String );
  
    addItemInfo( group, "10Model", i18n("Camera Model"),
                 QVariant::String );
    item = addItemInfo( iptc, "11Dpi", i18n("DPI"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable );
    item = addItemInfo( iptc, "12IPTCCaption", i18n("Caption"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable | KFileMimeTypeInfo::MultiLine );
    item = addItemInfo( iptc, "13IPTCKeywords", i18n("Keywords"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable | KFileMimeTypeInfo::SqueezeText);
    item = addItemInfo( iptc, "14IPTCCredit", i18n("Credit"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable | KFileMimeTypeInfo::SqueezeText);
    item = addItemInfo( iptc, "15IPTCCopyright", i18n("Copyright"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable | KFileMimeTypeInfo::SqueezeText);
    item = addItemInfo( iptc, "16IPTCCity", i18n("City"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "17IPTCState", i18n("Province/State"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "18IPTCCountry", i18n("Country"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "19IPTCSpecial", i18n("Special Instructions"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "20IPTCByline", i18n("Byline"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "21IPTCCategory", i18n("Category (NAARTNDA)"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "22IPTCHeadline", i18n("Headline"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
    item = addItemInfo( iptc, "23IPTCSource", i18n("Source"), QVariant::String);
    setAttributes( item, KFileMimeTypeInfo::Modifiable | KFileMimeTypeInfo::Addable);
  }
}

bool rawimagePlugin::readInfo( KFileMetaInfo& info, uint what) {
  KFileMetaInfoGroup group = appendGroup(info, "INFO");
  KFileMetaInfoGroup iptc  = appendGroup(info, "IPTCHeader");
  const QString path(info.path());
  if ( path.isEmpty() ) return false; // remote file

  // setup Perl program
  char *embedding[] = {"", "-e", "0"};
  QString exiftool = "my $file=\"" + QFile::encodeName(path) + "\";"
                     "use Image::ExifTool;my $exiftool = new Image::ExifTool;"
                     "$exiftool->Options(PrintConv => 1);$exiftool->Options(Binary => 0);"
                     "my $info = $exiftool->ImageInfo($file);"
                     "$return01=$$info{\'Make\'};"
                     "$return02=$$info{\'Model\'};"
                     "$return03=$$info{\'FocalLength35efl\'};"
                     "if ($return03 == \"\") {$return03=$$info{\'FocalLength\'};}"
                     "$return04=$$info{\'Aperture\'};"
                     "$return05=$$info{\'ShutterSpeedValue\'};"
                     "if ($return05 == \"\") {$return05=$$info{\'ShutterSpeed\'};}"
                     "$return05=$return05.\"s\";"
                     "$return06=$$info{\'MaxAperture\'};"
                     "if ($return06 != \"\") {$return06=$$info{\'Lens\'}.\"/\".$return06;} else {$return06=$$info{\'Lens\'};}"
                     "$return07=$$info{\'DateCreated\'};"
                     "if ($return07 == \"\") {$return07=$$info{\'DateTimeOriginal\'};}"
                     "if ($return07 == \"\") {$return07=$$info{\'CreateDate\'};}"
                     "$return08=$$info{\'ISO\'};"
                     "$return09=$$info{\'Flash\'};"
                     "if ($return09 == \"\") {$return09=$$info{\'FlashOn\'};}"
                     "$return10=$$info{\'ExifImageWidth\'};"
                     "if ($return10 == \"\") {$return10=$$info{\'ImageWidth\'};}"
                     "$return11=$$info{\'ExifImageLength\'};"
                     "if ($return11 == \"\") {$return11=$$info{\'ImageHeight\'};}"
                     "$return12=$$info{\'XResolution\'};"
                     "$return13=$$info{\'Caption-Abstract\'};"
                     "$return14=$$info{\'Keywords\'};"
                     "$return15=$$info{\'Credit\'};"
                     "$return16=$$info{\'CopyrightNotice\'};"
                     "$return17=$$info{\'City\'};"
                     "$return18=$$info{\'Province-State\'};"
                     "$return19=$$info{\'Country-PrimaryLocationName\'};"
                     "$return20=$$info{\'SpecialInstructions\'};"
                     "$return21=$$info{\'By-line\'};"
                     "$return22=$$info{\'Category\'};"
                     "$return23=$$info{\'Headline\'};"
                     "$return24=$$info{\'Source\'};"
                     "if ($return24 == \"\") {$return24=$$info{\'DocumentName\'};}"
                     "if ($return24 == \"\") {$return24=$$info{\'FileName\'};}";


  //kdDebug(7034) << "rawimagePlugin::readInfo: "<< exiftool << endl;
  // run Perl program
  my_perl = perl_alloc();
  // sequentially use of static Perl interpreter need this
  // ahead of every perl_construct/perl_destruct
  PL_perl_destruct_level = 1;
  perl_construct( my_perl );
  perl_parse(my_perl, xs_init, 3, embedding, (char **)NULL);
  PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
  perl_run(my_perl);
  perl_eval_pv((char*) exiftool.latin1(), TRUE);
  STRLEN n_a;
  QString make    = SvPV(perl_get_sv("return01", FALSE), n_a);
  QString model   = SvPV(perl_get_sv("return02", FALSE), n_a);
  QString focal   = SvPV(perl_get_sv("return03", FALSE), n_a);
  QString aperture= SvPV(perl_get_sv("return04", FALSE), n_a);
  QString exptime = SvPV(perl_get_sv("return05", FALSE), n_a);
  QString lens    = SvPV(perl_get_sv("return06", FALSE), n_a);
  QString create  = SvPV(perl_get_sv("return07", FALSE), n_a);
  QString iso     = SvPV(perl_get_sv("return08", FALSE), n_a);
  QString flash   = SvPV(perl_get_sv("return09", FALSE), n_a);
  
  if (flash == "On" || flash == "fired") flash = i18n("Flash", "fired");
  else flash = i18n("Flash", "No");
  QString size    = SvPV(perl_get_sv("return10", FALSE), n_a);
  size+="x";
  size+=SvPV(perl_get_sv("return11", FALSE), n_a);
  QString dpi           = SvPV(perl_get_sv("return12", FALSE), n_a);
  QString IPTCcaption   = SvPV(perl_get_sv("return13", FALSE), n_a);
  QString IPTCkeywords  = SvPV(perl_get_sv("return14", FALSE), n_a);
  QString IPTCcredit    = SvPV(perl_get_sv("return15", FALSE), n_a);
  QString IPTCcopyright = SvPV(perl_get_sv("return16", FALSE), n_a);
  QString IPTCcity      = SvPV(perl_get_sv("return17", FALSE), n_a);
  QString IPTCstate     = SvPV(perl_get_sv("return18", FALSE), n_a);
  QString IPTCcountry   = SvPV(perl_get_sv("return19", FALSE), n_a);
  QString IPTCspecial   = SvPV(perl_get_sv("return20", FALSE), n_a);
  QString IPTCbyline    = SvPV(perl_get_sv("return21", FALSE), n_a);
  QString IPTCcategory  = SvPV(perl_get_sv("return22", FALSE), n_a);
  QString IPTCheadline  = SvPV(perl_get_sv("return23", FALSE), n_a);
  QString IPTCsource    = SvPV(perl_get_sv("return24", FALSE), n_a);

  PL_perl_destruct_level = 1;
  perl_destruct(my_perl);
  perl_free(my_perl);
  PERL_SYS_TERM();

  // and finally display it!
  if (create.length()>0)    appendItem(group, "01CreationDate", create);
  if (focal.length()>0)     appendItem(group, "02FocalLength", focal);
  if (aperture.length()>0)  appendItem(group, "03Aperture", aperture);
  if (exptime.length()>1)   appendItem(group, "04ExposureTime", exptime);
  if (lens.length()>1)      appendItem(group, "05LensType", lens);
  appendItem(group, "06Iso", iso);
  appendItem(group, "07FlashUsed", flash);
  appendItem(group, "08Dimensions", size);
  appendItem(group, "09Manufacturer", make);
  appendItem(group, "10Model", model);
  appendItem(iptc, "11Dpi", dpi);
  appendItem(iptc, "22IPTCHeadline", IPTCheadline);
  appendItem(iptc, "12IPTCCaption", IPTCcaption);
  appendItem(iptc, "13IPTCKeywords", IPTCkeywords);
  appendItem(iptc, "14IPTCCredit", IPTCcredit);
  appendItem(iptc, "15IPTCCopyright", IPTCcopyright);
  appendItem(iptc, "16IPTCCity", IPTCcity);
  appendItem(iptc, "17IPTCState", IPTCstate);
  appendItem(iptc, "18IPTCCountry", IPTCcountry);
  appendItem(iptc, "19IPTCSpecial", IPTCspecial);
  appendItem(iptc, "20IPTCByline", IPTCbyline);
  appendItem(iptc, "21IPTCCategory", IPTCcategory);
  appendItem(iptc, "23IPTCSource", IPTCsource);

  return true;
}

bool rawimagePlugin::writeInfo( const KFileMetaInfo& info ) const {
  const QString source(info.path());
  if ( source.isEmpty() ) return false; // remote file
  kdDebug(7034) << "rawimagePlugin::writeInfo: "<< source << endl;

  KTempFile tempfile;
  tempfile.close();
  const QString dest(tempfile.name()); // temp-file for exiftool
  tempfile.unlink(); // do not let KDE control this file, its for Perl
  kdDebug(7034) << "rawimagePlugin::writeInfo: "<< dest << endl;

  QString IPTCcreateddate = info["INFO"].value("01CreationDate").toString();
  QString dpi = info["IPTCHeader"].value("11Dpi").toString();
  QString IPTCcaption = info["IPTCHeader"].value("12IPTCCaption").toString();
  QString IPTCkeywords = info["IPTCHeader"].value("13IPTCKeywords").toString();
  QString IPTCcredit = info["IPTCHeader"].value("14IPTCCredit").toString();
  QString IPTCcopyright = info["IPTCHeader"].value("15IPTCCopyright").toString();
  QString IPTCcity = info["IPTCHeader"].value("16IPTCCity").toString();
  QString IPTCstate = info["IPTCHeader"].value("17IPTCState").toString();
  QString IPTCcountry = info["IPTCHeader"].value("18IPTCCountry").toString();
  QString IPTCspecial = info["IPTCHeader"].value("19IPTCSpecial").toString();
  QString IPTCbyline = info["IPTCHeader"].value("20IPTCByline").toString();
  QString IPTCcategory = info["IPTCHeader"].value("21IPTCCategory").toString();
  QString IPTCheadline = info["IPTCHeader"].value("22IPTCHeadline").toString();
  QString IPTCsource = info["IPTCHeader"].value("23IPTCsource").toString();

  // setup Perl program
  char* embedding[] = {"", "-e", "0"};
  QString exiftool;
  exiftool  = "my $source=\"" + QFile::encodeName(source) + "\";"
              "my $dest=\"" + QFile::encodeName(dest) + "\";"
              "use Image::ExifTool; my $exiftool = new Image::ExifTool;";
  if (IPTCcreateddate.length()>0) {
      exiftool += "$exiftool->SetNewValue(\'IPTC:DateCreated\' => \'"+IPTCcreateddate+"\');";
  }
  if (dpi.length()>0) {
    exiftool += "$exiftool->SetNewValue(\'XResolution\' =>" + dpi + ");"
                "$exiftool->SetNewValue(\'YResolution\' =>" + dpi + ");";
  }
  if (IPTCcaption.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Caption-Abstract\' => \'"+IPTCcaption+"\');";
  }
  if (IPTCkeywords.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Keywords\' => \'"+IPTCkeywords+"\');";
  }
  if (IPTCcredit.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Credit\' => \'"+IPTCcredit+"\');";
  }
  if (IPTCcopyright.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:CopyrightNotice\' => \'"+IPTCcopyright+"\');";
  }
  if (IPTCcity.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:City\' => \'"+IPTCcity+"\');";
  }
  if (IPTCstate.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Province-State\' => \'"+IPTCstate+"\');";
  }
  if (IPTCcountry.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Country-PrimaryLocationName\' => \'"+IPTCcountry+"\');";
  }
  if (IPTCspecial.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:SpecialInstructions\' => \'"+IPTCspecial+"\');";
  }
  if (IPTCbyline.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:By-line\' => \'"+IPTCbyline+"\');";
  }
  if (IPTCcategory.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Category\' => \'"+IPTCcategory+"\');";
  }
  if (IPTCheadline.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Headline\' => \'"+IPTCheadline+"\');";
  }
  if (IPTCsource.length() > 0) {
    exiftool += "$exiftool->SetNewValue(\'IPTC:Source\' => \'"+IPTCsource+"\');";
  }
  exiftool += "$return=$exiftool->WriteInfo($source, $dest);";

  //kdDebug(7034) << "rawimagePlugin::writeInfo: "<< endl << exiftool << endl;

  // run Perl program
  my_perl = perl_alloc();
  PL_perl_destruct_level = 1;
  perl_construct( my_perl );
  perl_parse(my_perl, xs_init, 3, embedding, (char **)NULL);
  PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
  perl_run(my_perl);

  perl_eval_pv((char*) exiftool.latin1(), TRUE);
  int status=SvIV(perl_get_sv("return", FALSE));

  PL_perl_destruct_level = 1;
  perl_destruct(my_perl);
  perl_free(my_perl);
  PERL_SYS_TERM();

  if (status > 0) {
    QFile rfile( dest );
    rfile.open( IO_ReadOnly );
    QDataStream rstream( &rfile );
    QByteArray membuf( rfile.size() );
    rstream.readRawBytes( membuf.data(), membuf.size() );
    rfile.close(); 
    rfile.remove();
    QFile wfile( source );
    wfile.open( IO_WriteOnly );
    QDataStream wstream( &wfile );
    wstream.writeRawBytes( membuf.data(), membuf.size() );
    wfile.close();
  }
  if (status == 0) {
    QFile rfile( dest );
    rfile.remove();
    return false;
  }
  return true;
}

#include "kfile_rawimage.moc"

