/*  This file is part of the KDE RAW Image Thumbnail
    Copyright (C) 2000 Carsten Pfeiffer <pfeiffer@kde.org>
    Copyright (C) 2000 Malte Starostik <malte@kde.org>
    Copyright (C) 2005 Thorsten Schnebeck <thorsten.schnebeck@gmx.net>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

    This code uses Image::ExifTool (http://www.sno.phy.queensu.ca/~phil/exiftool)
*/

#include <assert.h>
#include <qimage.h>
#include <kimageio.h>
#include <qfile.h>
#include <qwmatrix.h>
#include "rawimagecreator.h"

extern "C" {
  #include <INTERN.h>
  #include <perl.h>
  static PerlInterpreter *my_perl;

  KDE_EXPORT ThumbCreator *new_creator() {
    KImageIO::registerFormats();
    return new RawImageCreator;
  }
}

bool RawImageCreator::create(const QString &path, int width, int height, QImage &img) {
  // create image preview

  // setup Perl program
  int orientation;
  STRLEN n_a;
  char *embedding[] = {"", "-e", "0"};
  QString exiftool = "my $file=\"" + QFile::encodeName(path) + "\";"
                     "use Image::ExifTool;my $exiftool = new Image::ExifTool;"
                     "$exiftool->Options(PrintConv => 0);$exiftool->Options(Binary => 1);"
                     "$info = $exiftool->ImageInfo($file);"
                     "my $val=$$info{\'JpgFromRaw\'};"
                     "if ($val == \"\") {$val=$$info{\'PreviewImage\'};"
                     "if ($val == \"\") {$val=$$info{\'ThumbnailImage\'};};};"
                     "$return0=$$val;"
                     "$return1=$$info{\'Orientation\'};"
                     "if ($return1 == \"\") {$return1 = $$info{\'Rotation\'};}";
  // run Perl program
  my_perl = perl_alloc();
  perl_construct( my_perl );
  perl_parse(my_perl, NULL, 3, embedding, (char **)NULL);
  PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
  perl_run(my_perl);
  perl_eval_pv((char*) exiftool.latin1(), TRUE);
  const uchar* mypic = (uchar*) SvPV(perl_get_sv("return0", FALSE), n_a);
  img.loadFromData(mypic, n_a);
  if (!img.isNull()) {
    img.smoothScale(width, height, QImage::ScaleMax);
    orientation=SvIV(perl_get_sv("return1", FALSE));
    switch (orientation) {
      case 0: case 360:
        orientation=1;
        break;
      case 90:
        orientation=6;
        break;
      case 180:
        orientation=3;
        break;
      case 270:
        orientation=8;
        break;
    }
  }
  perl_destruct(my_perl);
  perl_free(my_perl);
  PERL_SYS_TERM();

  if( img.isNull() ) return false;
  QWMatrix M;
  QWMatrix flip= QWMatrix(-1,0,0,1,0,0);
  switch(orientation) {
    case 2: M = flip; break;
    case 4: M = flip;
    case 3: M.rotate(180); break;
    case 5: M = flip;
    case 6: M.rotate(90); break;
    case 7: M = flip;
    case 8: M.rotate(270); break;
  }
  img = img.xForm(M);
  if (img.depth() != 32) img = img.convertDepth(32);
  return true;
}

ThumbCreator::Flags RawImageCreator::flags() const {
  return None;
}
